@@ -28,6 +28,7 @@
 
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
@@ -36,7 +37,6 @@
 #include <libgnome/gnome-url.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-authentication-manager.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <profiles/gnome-media-profiles.h>
 #include <gst/gst.h>
 
@@ -54,7 +54,6 @@
 #include "sj-play.h"
 #include "sj-genres.h"
 #include "gedit-message-area.h"
-#include "bacon-volume.h"
 
 gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
 
@@ -76,7 +75,7 @@ SjExtractor *extractor;
 GConfClient *gconf_client;
 
 GtkWidget *main_window;
-static GtkWidget *message_area_vbox;
+static GtkWidget *message_area_eventbox;
 static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry, *year_entry, *disc_number_entry;
 static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
@@ -90,7 +89,7 @@ GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
 GtkWidget *current_message_area;
 
 const char *path_pattern, *file_pattern;
-char *base_uri;
+GFile *base_uri;
 NautilusBurnDrive *drive = NULL;
 gboolean strip_chars;
 gboolean eject_finished;
@@ -402,9 +401,7 @@ set_message_area (GtkWidget *container,
   if (message_area == NULL)
     return;
 
-  gtk_box_pack_start (GTK_BOX (container),
-                      current_message_area,
-                      FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (container), message_area);
 
   g_object_add_weak_pointer (G_OBJECT (current_message_area),
                              (gpointer)&current_message_area);
@@ -451,7 +448,7 @@ musicbrainz_submit_message_area_response (GeditMessageArea *message_area,
     on_submit_activate (NULL, NULL);
   }
   
-  set_message_area (message_area_vbox, NULL);
+  set_message_area (message_area_eventbox, NULL);
 }
 
 /**
@@ -490,7 +487,7 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (next_menuitem, FALSE);
     set_duplication (FALSE);
 
-    set_message_area (message_area_vbox, NULL);
+    set_message_area (message_area_eventbox, NULL);
   } else {
     gtk_list_store_clear (track_store);
 
@@ -561,7 +558,7 @@ static void update_ui_for_album (AlbumDetails *album)
 
       message_area = musicbrainz_submit_message_area_new (album->title, album->artist);
 
-      set_message_area (message_area_vbox, message_area);
+      set_message_area (message_area_eventbox, message_area);
 
       g_signal_connect (message_area,
                         "response",
@@ -688,11 +685,13 @@ AlbumDetails* multiple_album_dialog(GList *albums)
 static void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   g_assert (strcmp (entry->key, GCONF_BASEURI) == 0);
-  g_free (base_uri);
+  if (base_uri) {
+    g_object_unref (base_uri);
+  }
   if (entry->value == NULL) {
     base_uri = sj_get_default_music_directory ();
   } else {
-    base_uri = g_strdup (gconf_value_get_string (entry->value));
+    base_uri = g_file_new_for_uri (gconf_value_get_string (entry->value));
   }
   /* TODO: sanity check the URI somewhat */
 }
@@ -791,9 +790,9 @@ static void audio_volume_changed_cb (GConfClient *client, guint cnxn_id, GConfEn
   
   GtkWidget *volb = glade_xml_get_widget (glade, "volume_button");
   if (entry->value == NULL) {
-    bacon_volume_button_set_value (BACON_VOLUME_BUTTON (volb), 1.0);
+    gtk_scale_button_set_value (GTK_SCALE_BUTTON (volb), 1.0);
   } else {
-    bacon_volume_button_set_value (BACON_VOLUME_BUTTON (volb), gconf_value_get_float (entry->value));
+    gtk_scale_button_set_value (GTK_SCALE_BUTTON (volb), gconf_value_get_float (entry->value));
   }
 }
 
@@ -885,6 +884,9 @@ static void reread_cd (gboolean ignore_no_media)
   GdkCursor *cursor;
   gboolean realized = GTK_WIDGET_REALIZED (main_window);
   
+  /* Make sure nothing is playing */
+  stop_playback ();
+  
   /* Set watch cursor */
   if (realized) {
     cursor = gdk_cursor_new_for_display (gdk_drawable_get_display (main_window->window), GDK_WATCH);
@@ -1468,15 +1470,6 @@ void on_contents_activate(GtkWidget *button, gpointer user_data) {
   }
 }
 
-GtkWidget *
-sj_make_volume_button (void)
-{
-  GtkWidget *w = bacon_volume_button_new (GTK_ICON_SIZE_SMALL_TOOLBAR,
-                                          0.0, 1.0, 0.02);
-  bacon_volume_button_set_value (BACON_VOLUME_BUTTON (w), 1.0);
-  return w;
-}
-
 static void
 upgrade_gconf (void)
 {
@@ -1485,12 +1478,15 @@ upgrade_gconf (void)
   if (s != NULL) {
     g_free (s);
   } else {
+    GFile *gfile;
     char *uri;
     s = gconf_client_get_string (gconf_client, GCONF_BASEPATH, NULL);
     if (s == NULL)
       return;
-    uri = gnome_vfs_get_uri_from_local_path (s);
+    gfile = g_file_new_for_path (s);
+    uri = g_file_get_uri (gfile);
     g_free (s);
+    g_object_unref (gfile);
     gconf_client_set_string (gconf_client, GCONF_BASEURI, uri, NULL);
     g_free (uri);
   }
@@ -1551,7 +1547,7 @@ void on_duplicate_activate (GtkWidget *button, gpointer user_data)
   const gchar* device;
 
   device = nautilus_burn_drive_get_device (drive);
-  if (!g_spawn_command_line_sync (g_strconcat ("nautilus-cd-burner --source-device=", device, NULL), NULL, NULL, NULL, &error)) {
+  if (!g_spawn_command_line_async (g_strconcat ("nautilus-cd-burner --source-device=", device, NULL), &error)) {
       GtkWidget *dialog;
 
       dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
@@ -1586,12 +1582,13 @@ int main (int argc, char **argv)
   GnomeProgram *program;
   GError *error = NULL;
   GtkTreeSelection *selection;
-  char *device = NULL;
+  char *device = NULL, **uris = NULL;
   GOptionContext *ctx;
   const GOptionEntry entries[] = {
     { "auto-start", 'a', 0, G_OPTION_ARG_NONE, &autostart, N_("Start extracting immediately"), NULL },
     { "play", 'p', 0, G_OPTION_ARG_NONE, &autoplay, N_("Start playing immediately"), NULL},
     { "device", 'd', 0, G_OPTION_ARG_FILENAME, &device, N_("What CD device to read"), N_("DEVICE") },
+    { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &uris, N_("URI to the CD device to read"), NULL },
     { NULL }
   };
 
@@ -1680,7 +1677,7 @@ int main (int argc, char **argv)
   glade_xml_signal_autoconnect (glade);
 
   main_window = glade_xml_get_widget (glade, "main_window");
-  message_area_vbox = glade_xml_get_widget (glade, "message_area_vbox");
+  message_area_eventbox = glade_xml_get_widget (glade, "message_area_eventbox");
   select_all_menuitem = glade_xml_get_widget (glade, "select_all");
   deselect_all_menuitem = glade_xml_get_widget (glade, "deselect_all");
   submit_menuitem = glade_xml_get_widget (glade, "submit");
@@ -1813,10 +1810,28 @@ int main (int argc, char **argv)
   eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
   open_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_OPEN, NULL, TRUE, NULL), NULL);
   audio_volume_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_VOLUME, NULL, TRUE, NULL), NULL);
-  if (device == NULL) {
+  if (device == NULL && uris == NULL) {
     device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), GINT_TO_POINTER (TRUE));
   } else {
-    set_device (device, TRUE);
+    if (device)
+      set_device (device, TRUE);
+    else {
+      char *d;
+
+      /* Mash up the CDDA URIs into a device path */
+      if (g_str_has_prefix (uris[0], "cdda://")) {
+      	gint len;
+        d = g_strdup_printf ("/dev/%s%c", uris[0] + strlen ("cdda://"), '\0');
+        /* Take last '/' out of path, or set_device thinks it is part of the device name */
+		len = strlen (d);
+		if (d[len - 1] == '/')
+			d [len - 1] = '\0';
+	set_device (d, TRUE);
+	g_free (d);
+      } else {
+        device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), GINT_TO_POINTER (TRUE));
+      }
+    }
   }
 
   if (sj_extractor_supports_encoding (&error) == FALSE) {
@@ -1834,7 +1849,7 @@ int main (int argc, char **argv)
 
   nautilus_burn_shutdown ();
 
-  g_free (base_uri);
+  g_object_unref (base_uri);
   g_object_unref (metadata);
   g_object_unref (extractor);
   g_object_unref (gconf_client);
